require 'spec_helper'

describe 'getting all documents' do
  let(:document_jsons) { (1..27).map{ |i| Hashie::Mash.new(id: "id#{i}", _score: 1.0, name: "doc#{i}") } }
    
  def link page, per_page
    "https://us-east-1.s3searchapp.test/v1/documents.json?page=#{page}&per_page=#{per_page}"
  end  

  it 'takes per_page and page params' do
    stub_get_all 2, 3
    results = S3Search.get_all page: 2, per_page: 3
    expect_page(results, 2, 3)

    expect(results.map(&:name)).to eq(['doc4', 'doc5', 'doc6'])
  end

  it 'allows paging with next and prev' do
    stub_get_all 4, 2
    page = S3Search.get_all page: 4, per_page: 2
    expect_page(page, 4, 2)

    stub_get_all 5, 2
    page.next
    expect_page(page, 5, 2)

    page.prev
    expect_page(page, 4, 2)

    stub_get_all 3, 2
    page.prev
    expect_page(page, 3, 2)

    stub_get_all 2, 2
    page.prev
    expect_page(page, 2, 2)

    stub_get_all 1, 2
    page.prev
    expect_page(page, 1, 2)
    
    page.prev
    expect_page(page, 1, 2)
  end

  it 'defaults to page: 1, per_page: 25' do
    stub_get_all 1, 25
    page = S3Search.get_all
    expect_page(page, 1, 25)

    stub_get_all 2, 25
    page.next
    expect_page(page, 2, 25)

    page.next
    expect_page(page, 2, 25)
  end

  def expect_page result, page, per_page
    from = from(page, per_page)
    to = to(page, per_page)

    expect(result.size).to eq((to - from) + 1)
    expect(result.from).to eq(from)
    expect(result.to).to eq(to)
    expect(result.first).to eq(document_jsons[from - 1])
    expect(result.last).to eq(document_jsons[to - 1])
  end


  def from page, per_page
    [(page - 1) * per_page + 1, 1].max
  end

  def to page, per_page, total=27
    [page * per_page, total].min
  end
  
  def stub_get_all page, per_page
    total = 27
    from = from(page, per_page)
    to = to(page, per_page)
    links = {
      self: link(page, per_page)
    }
    links[:next] = link(page + 1, per_page) if to < total
    links[:prev] = link(page - 1, per_page) if page > 1

    stub_api_http :get, "/v1/documents.json?page=#{page}&per_page=#{per_page}" do
      {
        body: Hashie::Mash.new(
          total: total,
          from: from,
          to: to,
          results: document_jsons[(from - 1)..(to - 1)],
          links: links
        ),
        status: 200
      }
    end
  end
end
